#pragma once

#include "framework.h"

#define LIBRARY_VERSION "1.0.2"

#define REG_DMA_DATA_BASE           0x1000
#define REG_DMA_INDEX               0x1040
#define REG_DMA_DISABLE             0x1044
#define REG_INTERRUPT               0x100C
#define REG_INTERRUPT_MASK          0x1010
#define REG_CTR_TIMEOUT_ABSOLUTE    0x1050
#define REG_CTR_TIMEOUT_INTERVAL    0x1060
#define REG_DATA_COUNTER            0x1070
#define REG_SC_OUT                  0x1080
#define REG_SC_IN                   0x1084
#define REG_SC_INT_MASK             0x1088

#define REG_LBL_CONF_PCI_0(numCh)   (0x1400 + numCh * 0x40) 
#define REG_LBL_CONF_PCI_1(numCh)   (0x1404 + numCh * 0x40) 
#define REG_LBL_CONF_PCI_2(numCh)   (0x1408 + numCh * 0x40) 
#define REG_LBL_CONF_PCI_3(numCh)   (0x140C + numCh * 0x40) 
#define REG_LBL_CONF_PCI_4(numCh)   (0x1410 + numCh * 0x40) 
#define REG_LBL_CONF_PCI_5(numCh)   (0x1414 + numCh * 0x40) 
#define REG_LBL_CONF_PCI_6(numCh)   (0x1418 + numCh * 0x40) 
#define REG_LBL_CONF_PCI_7(numCh)   (0x141C + numCh * 0x40) 
#define REG_RX_CONF(numCh)          (0x1420 + numCh * 0x40) 

#define REG_TX_CONF(numCh)          (0x1428 + numCh * 0x40) 
#define REG_TX_CONTROL(numCh)       (0x142C + numCh * 0x40) 
#define REG_TX_FIFO(numCh)          (0x1430 + numCh * 0x40) 
#define REG_TX_DATA_RAM(numCh)      (0x8000 + numCh * 0x800)
#define REG_TX_DESC_RAM(numCh)      (0x8400 + numCh * 0x800)

#define REG_RX_CONF_708(numCh)      (0x1500 + numCh * 0x40)     ///<   708
#define REG_TX_CONF_708(numCh)      (0x1528 + numCh * 0x40)     ///<   708
#define REG_TX_TIME_708(numCh)      (0x152C + numCh * 0x40)     ///<    708

#define REG_TX708_DAT_BLOCK1(numCh) (0x9000 + numCh * 0x800)    ///<   1 ()  708
#define REG_TX708_DAT_BLOCK2(numCh) (0x9100 + numCh * 0x800)    ///<   2 ()  708
#define REG_TX708_DAT_BLOCK3(numCh) (0x9200 + numCh * 0x800)    ///<   3 ()  708
#define REG_TX708_DAT_BLOCK4(numCh) (0x9300 + numCh * 0x800)    ///<   4 ()  708
#define REG_TX708_DAT_BLOCK5(numCh) (0x9400 + numCh * 0x800)    ///<   5 ()  708
#define REG_TX708_DAT_BLOCK6(numCh) (0x9500 + numCh * 0x800)    ///<   6 ()  708
#define REG_TX708_DAT_BLOCK7(numCh) (0x9600 + numCh * 0x800)    ///<   7 ()  708
#define REG_TX708_DAT_BLOCK8(numCh) (0x9700 + numCh * 0x800)    ///<   8 ()  708

#define A708_MODE_0         0               ///<   0
#define A708_MODE_1         1               ///<   1
#define A708_MODE_2         2               ///<   2
#define A708_MODE_3         3               ///<   3

#define A708_STOP           0               ///< 
#define A708_START          ~A708_STOP      ///< 

#define A708_OFF            0               ///< 
#define A708_ON             ~A708_OFF       ///< 

#define A708_FALSE          0               ///< 
#define A708_TRUE           ~A708_FALSE     ///< 

#define A708_SPEED_100          0       ///< 100 /
#define A708_SPEED_12_14        1       ///< 12.5 /
#define A708_SPEED_50           2       ///< 50 /
#define A708_SPEED_12           3       ///< 12 /
#define A708_SPEED_14_5         4       ///< 14.5 /

#define A708_TXRR_10MS      0       ///< txrr 10 ms
#define A708_TXRR_1MS       1       ///< txrr 1 ms 

#define A708_SC_OUT_1           (1)         ///<    0
#define A708_SC_OUT_0           (1<<1)      ///<    1
#define A708_SC_OUT_NO_ACT      0           ///<    

#define A708_IN_RK_1    0       ///<   1
#define A708_IN_RK_2    1       ///<   2
#define A708_IN_RK_3    2       ///<   3
#define A708_IN_RK_4    3       ///<   4
#define A708_IN_RK_5    4       ///<   4
#define A708_IN_RK_6    5       ///<   4
#define A708_IN_RK_7    6       ///<   4
#define A708_IN_RK_8    7       ///<   4

#define A708_SC_RISE    0                   ///<  
#define A708_SC_FALL    ~A708_SC_RISE       ///<  

/// \brief   708 - 
#define A708_TX708_MODE_OFF             0
/// \brief   708 -  
#define A708_TX708_MODE_SINGLE_SHOT     1
/// \brief   708 -  
#define A708_TX708_MODE_AUTO            3

#define ASSERT_A708(arinc) ((arinc)->channel_type == T_CHANNEL_708)
#define ASSERT_NOT_A708(arinc) ((arinc)->channel_type != T_CHANNEL_708)

/// \brief channel info
typedef struct sA708_CHANNEL_INFO {
	WCHAR* device_name;		///   
	UINT32 g_channel;		///   
	UINT32 i_channel;		///    
	UINT32 b_channel;		///    
	UINT32 g_board;			///  
	UINT32 channel_type;		///   (T_CHANNEL_RX  T_CHANNEL_TX)
	HANDLE arinc;			///  
} A708_CHANNEL_INFO, A708_DEVICE_INFO, *PA708_CHANNEL_INFO;

typedef struct A708_DEVICE_LIST {
	UINT32 count;				///  
	UINT32 pcount;				///  

	A708_CHANNEL_INFO* device;	///  
	A708_CHANNEL_INFO* pdev;	///  
} A708_DEVICE_LIST, * PA708_DEVICE_LIST;


#define A708_DATA_BLOCK_SIZE_708    50

/// \brief   708
typedef struct {
	UINT32 words[A708_DATA_BLOCK_SIZE_708];       ///<  
	int length;                                         ///< -  
} T_A708_DATA_BLOCK;

/**
* @brief   
*
*          ID GUID_DEVINTERFACE_ARINC429,
*  ,         -   . 
* A429_DEVICE_LIST      a429_free.
*
*
* @return    A429_DEVICE_LIST
*/
A708_DEVICE_LIST* a708_enumerate();

/**
* @brief  
*/
VOID a708_d();

/**
* @brief   
*
* @param deviecs    A429_DEVICE_LIST
*/
VOID a708_free(A708_DEVICE_LIST* devices);

/**
* @brief   
*
*             A429_CHANNEL_INFO.
*             CloseHandle.
*
* @param    A429_CHANNEL_INFO
*
* @return    NULL,   
*/
HANDLE a708_open(A708_CHANNEL_INFO* info);

/**
* @brief   
*
* @param arinc    
* @param addr 
* @param pbuf   UINT32,     
* @return    0   
*/
UINT32 a708_read_reg(A708_CHANNEL_INFO* arinc, UINT32 addr, UINT32* pbuf);

/**
* @brief   
*
* @param arinc    
* @param addr 
* @param buf  
* @return    0   
*/
UINT32 a708_write_reg(A708_CHANNEL_INFO* arinc, UINT32 addr, UINT32 buf);

/**
* @brief   
*
* @param arinc    
* @param addr 
* @param buf  
* @param mask 
* @return    0   
*/
UINT32 a708_modify_reg(A708_CHANNEL_INFO* arinc, UINT32 addr, UINT32 buf, UINT32 mask);

/**
* @brief  DMA
*
* @param arinc    
* @param channel   
* @param pcount      ,        
* @param dmabuf   ,     DMA
*
* @return    0   
*/
UINT32 a708_read_dma(A708_CHANNEL_INFO* arinc, UINT32* pcount, DMA_BLOCK** dmabuf);

/**
* @brief  DMA
*
*   DMA    
*
* @param arinc    
*
* @return    0   
*/
UINT32 a708_clear_dma(A708_CHANNEL_INFO* arinc);

/**
* @brief    
*
* @param arinc    
* @param version         
*
* @return    0   
*/
UINT32 a708_get_device_info(A708_CHANNEL_INFO* arinc, VERSION_REQ* version);

/**
* @brief  DMA 
*
* @param arinc    
* @param value A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a708_manageDma(A708_CHANNEL_INFO* arinc, UINT32 value);

/**
* @brief  DMA 
*
* @param arinc    
* @param value A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a708_manageChannelDma(A708_CHANNEL_INFO* arinc, UINT32 value);

/**
* @brief  RAM 
*
* @param arinc    
* @param container    RAM_REQ  ,     
*
* @return    0   
*/
UINT32 a708_writeRam(A708_CHANNEL_INFO* arinc, RAM_REQ* container);

/**
* @brief  RAM 
*
* @param arinc    
* @param container    RAM_REQ,     
*
* @return    0   
*/
UINT32 a708_readRam(A708_CHANNEL_INFO* arinc, RAM_REQ* container);

/**
* @brief     
*
* @param arinc    
* @param info    T_INT_REQ
*
* @return    0   
*/
UINT32 a708_manageTrackingInterrupt(A708_CHANNEL_INFO* arinc, T_INT_REQ* info);

/**
* @brief     
*
* @param arinc    
* @param info    T_INT_REQ,     
*
* @return    0   
*/
UINT32 a708_checkTrackingInterrupt(A708_CHANNEL_INFO* arinc, T_INT_REQ* info);

/**
* @brief    
*
* @param arinc    
* @param action A429_STOP  A429_START
*
* @return    0   
*/
UINT32 a708_txStartStop(A708_CHANNEL_INFO* arinc, UINT32 action);

/**
* @brief   
*
* @param arinc    
*
* @return    0   
*/
UINT32 a708_txStartOnce(A708_CHANNEL_INFO* arinc);

/**
* @brief    
*
* @param arinc    
* @param mode  
*
* @return    0   
*/
UINT32 a708_txSetMode(A708_CHANNEL_INFO* arinc, UINT32 mode);

/**
* @brief     FIFO 
*
* @param arinc    
* @param data  
*
* @return    0   
*/
UINT32 a708_writeTxFifo(A708_CHANNEL_INFO* arinc, UINT32 data);

/**
* @brief   
*
* @param arinc    
* @param numReg   REG_LBL_CONF_PCI_0 - REG_LBL_CONF_PCI_7
* @param mask 
* @param value 
*
* @return    0   
*/
UINT32 a708_setLabelConfReg(A708_CHANNEL_INFO* arinc, UINT32 numReg, UINT32 mask, UINT32 value);

/**
* @brief    
*
* @param arinc    
* @param speed  (A429_SPEED_...)
*
* @return    0   
*/
UINT32 a708_setSpeed(A708_CHANNEL_INFO* arinc, UINT32 speed);

/**
* @brief   
*
* @param arinc    
* @param action A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a708_manageEnBit(A708_CHANNEL_INFO* arinc, UINT32 action);

/**
* @brief    
*
* @param arinc    
* @param state   ,         - A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a708_isEnBit(A708_CHANNEL_INFO* arinc, UINT32* state);

/**
* @brief   
*
* @param arinc    
* @param parcheck  30  RX_CONF_REG  TX_CONF_REG
* @param parity  29  RX_CONF_REG  TX_CONF_REG
*
* @return    0   
*/
UINT32 a708_parityManage(A708_CHANNEL_INFO* arinc, UINT32 parcheck, UINT32 parity);

/**
* @brief    
*
* @param arinc    
* @param action A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a708_rxFiltration(A708_CHANNEL_INFO* arinc, UINT32 action);

/**
* @brief    
*
* @param arinc    
* @param label 
* @param action A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a708_rxManageFiltrationLabel(A708_CHANNEL_INFO* arinc, UINT32 label, UINT32 action);

/**
* @brief    
*
* @param arinc    
*
* @return    0   
*/
UINT32 a708_rxClearFiltrationLabel(A708_CHANNEL_INFO* arinc);

/**
* @brief   9  10 
*
* @param arinc    
* @param action      RCV_SDI
* @param bit9   9
* @param bit10   10
*
* @return    0   
*/
UINT32 a708_rxManageRcvSdi(A708_CHANNEL_INFO* arinc, UINT32 action, UINT32 bit9, UINT32 bit10);

/**
* @brief  
*
* @param arinc    
*
* @return    0   
*/
UINT32 a708_deinit(A708_CHANNEL_INFO* arinc);

/**
* @brief      
*
* @param arinc    
* @param action A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a708_manageReverce(A708_CHANNEL_INFO* arinc, UINT32 action);

/**
* @brief   
*
* @param arinc    
* @param gapBit  ,  19-25  REG_TX_CONF
*
* @return    0   
*/
UINT32 a708_txGapBits(A708_CHANNEL_INFO* arinc, UINT32 gapBit);

/**
* @brief    RRT
*
* @param arinc    
* @param txrr A429_TXRR_10MS  A429_TXRR_1MS
*
* @return    0   
*/
UINT32 a708_txRrD(A708_CHANNEL_INFO* arinc, UINT32 txrr);

/**
* @brief   RRT
*
* @param arinc    
* @param action A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a708_txSkipRrt(A708_CHANNEL_INFO* arinc, UINT32 action);

/**
* @brief    
*
* @param arinc    
*
* @return    0   
*/
UINT32 a708_clearScIntMask(A708_CHANNEL_INFO* arinc);

/**
* @brief   
*
* @param arinc    
* @param action1    1
* @param action2    2
* @param action3    3
* @param action4    4
*
* @return    0   
*/
UINT32 a708_manageScOut(A708_CHANNEL_INFO* arinc, UINT32 action1, UINT32 action2, UINT32 action3, UINT32 action4/*, UINT32 action5, UINT32 action6, UINT32 action7, UINT32 action8*/);

/**
* @brief    
*
* @param arinc    
* @param numRk	  
* @param stateRk A429_SC_RISE  A429_SC_FALL
* @param actionRk A429_OFF  A429_ON
*
* @return    0   
*/
UINT32 a708_manageScIntMask(A708_CHANNEL_INFO* arinc, UINT32 numRk, UINT32 stateRk, UINT32 actionRk);

/**
* @brief    RRT
*
* @param arinc    
* @param rrtValue   RRT
*
* @return    0   
*/
UINT32 a708_txStartWithRrt(A708_CHANNEL_INFO* arinc, UINT32 rrtValue);

/**
* @brief     RRT
*
* @param arinc    
* @param rrtValue   RRT
*
* @return    0   
*/
UINT32 a708_txStartWithRrtOnce(A708_CHANNEL_INFO* arinc, UINT32 rrtValue);

/**
* @brief      
*
* @param arinc    
* @param rknum   
*
* @return    0   
*/
UINT32 a708_txSetRequestTransferByRkIn(A708_CHANNEL_INFO* arinc, UINT32 rknum);

/**
* @brief   
*
* @param arinc    
* @param mask 
* @param value 
*
* @return    0   
*/
UINT32 a708_manageInterruptMask(A708_CHANNEL_INFO* arinc, UINT32 mask, UINT32 value);

/**
* @brief     
*
* @param arinc    
* @param rknum   
*
* @return    0   
*/
UINT32 a708_rxSetReceiveReadyByRkIn(A708_CHANNEL_INFO* arinc, UINT32 rknum);

/**
* @brief  RAM 
*
* @param arinc    
*
* @return    0   
*/
UINT32 a708_ramCls(A708_CHANNEL_INFO* arinc);


/// --- 708 ---

/**
* @brief      Arinc708
*
* @param arinc    
* @param mode A708_ON  A708_OFF
*
* @return    0   
*/
UINT32 a708_rx708_manage(A708_CHANNEL_INFO* arinc, UINT32 mode);

/**
* @brief      Arinc708
*
* @param arinc    
* @param mode A708_ON  A708_OFF
*
* @return    0   
*/
UINT32 a708_tx708_setMode(A708_CHANNEL_INFO* arinc, UINT32 mode);

/**
* @brief    ShortDMA   Arinc708
*
* @param arinc    
* @param mode A708_ON  A708_OFF
*
* @return    0   
*/
UINT32 a708_rx708_shortDma(A708_CHANNEL_INFO* arinc, UINT32 mode);

/**
* @brief   DMA  Arinc708
*
* @param arinc    
* @param pcount      (    )
* @param dmabuf    ,     DMA
*
* @return    0   
*/
UINT32 a708_readDmaA708(A708_CHANNEL_INFO* arinc, UINT32* pcount, DMA_BLOCK_708** dmabuf);

/**
* @brief     Arinc708
*
* @param arinc    
* @param numBlock     (  1)
* @param block      
*
* @return    0   
*/
UINT32 a708_tx708_writeDataBlock(A708_CHANNEL_INFO* arinc, UINT32 numBlock, T_A708_DATA_BLOCK* block);

/**
* @brief     Arinc708
*
* @param arinc    
* @param numBlock     (  1)
* @param block        
*
* @return    0   
*/
UINT32 a708_tx708_readDataBlock(A708_CHANNEL_INFO* arinc, UINT32 numBlock, T_A708_DATA_BLOCK* block);

/**
* @brief   Arinc708
*
* @param arinc    
* @param mode  , A708_TX708_MODE_SINGLE_SHOT  A708_TX708_MODE_AUTO
*
* @return    0   
*/
UINT32 a708_tx708_start(A708_CHANNEL_INFO* arinc, UINT32 mode);

/**
* @brief     Arinc708
*
* @param arinc    
* @param block   (  1)
* @param result   , A708_FALSE     ,  A708_TRUE
*
* @return    0   
*/
UINT32 a708_tx708_hasSendedDataBlock(A708_CHANNEL_INFO* arinc, UINT32 block, UINT32* result);

/**
* @brief   Arinc708
*
* @param arinc    
*
* @return    0   
*/
UINT32 a708_tx708_stop(A708_CHANNEL_INFO* arinc);

/**
* @brief   Arinc708
*
* @param arinc    
* @param value  
*
* @return    0   
*/
UINT32 a708_tx708_writeTimer(A708_CHANNEL_INFO* arinc, UINT32 value);