// libarinc708.cpp : Defines the functions for the static library.
//

#include "pch.h"
#include "libarinc708dll.h"

VOID a708_d() {
	printf("libarinc708dll %s built on %s %s\n", LIBRARY_VERSION, __DATE__, __TIME__);
}

A708_DEVICE_LIST* a708_enumerate()
{
	HDEVINFO        hdi;
	SP_DEVINFO_DATA DevinfoData;
	SP_DEVICE_INTERFACE_DATA DeviceInterfaceData;
	PSP_DEVICE_INTERFACE_DETAIL_DATA pDeviceInterfaceDetail;
	UINT32 i, channel_count = 0;
	UINT8 deviceCount = 0, devIndex = 0;
	ULONG size;
	A708_DEVICE_LIST* devices;
	A708_CHANNEL_INFO* chlist1 = NULL, * chlist2;
	DWORD wsize, errc;
	HANDLE handle;
	VERSION_REQ ver;

	devices = new A708_DEVICE_LIST;
	devices->count = 0;

	hdi = SetupDiGetClassDevs(&GUID_DEVINTERFACE_ARINC708, NULL, NULL, DIGCF_DEVICEINTERFACE);
	if (hdi == INVALID_HANDLE_VALUE) {
		SetupDiDestroyDeviceInfoList(hdi);
		return devices;
	}

	DeviceInterfaceData.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);
	while (SetupDiEnumDeviceInterfaces(hdi, NULL, &GUID_DEVINTERFACE_ARINC708, devices->count++, &DeviceInterfaceData));
	devices->count--;

	if (devices->count == 0) {
		SetupDiDestroyDeviceInfoList(hdi);
		return devices;
	}

	devices->pdev = (A708_CHANNEL_INFO*)malloc(sizeof(A708_CHANNEL_INFO) * devices->count);
	devices->device = (A708_CHANNEL_INFO*)malloc(sizeof(A708_CHANNEL_INFO) * devices->count);

	while (SetupDiEnumDeviceInterfaces(hdi, NULL, (LPGUID)&GUID_DEVINTERFACE_ARINC708, devIndex, &DeviceInterfaceData)) {

		SetupDiGetDeviceInterfaceDetail(hdi, &DeviceInterfaceData, NULL, 0, &size, NULL);
		if (GetLastError() != ERROR_INSUFFICIENT_BUFFER) {
			SetupDiDestroyDeviceInfoList(hdi);
			return devices;
		}

		pDeviceInterfaceDetail = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(size);
		pDeviceInterfaceDetail->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
		SetupDiGetDeviceInterfaceDetail(hdi, &DeviceInterfaceData, pDeviceInterfaceDetail, size, NULL, &DevinfoData);


		wsize = wcslen(pDeviceInterfaceDetail->DevicePath) + 1;
		devices->device[deviceCount].device_name = (WCHAR*)malloc(sizeof(WCHAR) * wsize);
		devices->pdev[deviceCount].device_name = (WCHAR*)malloc(sizeof(WCHAR) * wsize);
		wcsncpy_s(devices->device[deviceCount].device_name, wsize, pDeviceInterfaceDetail->DevicePath, wsize);
		wcsncpy_s(devices->pdev[deviceCount].device_name, wsize, pDeviceInterfaceDetail->DevicePath, wsize);
		free(pDeviceInterfaceDetail);
		devIndex++;

		handle = a708_open(devices->device + deviceCount);
		if (handle == INVALID_HANDLE_VALUE) {
			//printf("Device %d %ws can not be opened, does it exist\n", devIndex, devices->device[deviceCount].device_name);
			continue;
		}

		errc = a708_get_device_info(devices->device + deviceCount, &ver);

		if (errc == ERROR_SUCCESS) {
			chlist2 = (A708_CHANNEL_INFO*)malloc(sizeof(A708_CHANNEL_INFO) * (ver.rx_channel_count + ver.tx_channel_count + ver.a708_channel_count + channel_count));
			if (channel_count > 0) {
				memcpy(chlist2, chlist1, sizeof(A708_CHANNEL_INFO) * (channel_count));
				free(chlist1);
			}
			for (i = channel_count; i < channel_count + ver.rx_channel_count + ver.tx_channel_count + ver.a708_channel_count; i++) {
				chlist2[i].g_board = deviceCount;

				if (i - channel_count < ver.rx_channel_count) {
					//rx channel
					chlist2[i].channel_type = T_CHANNEL_RX;
					chlist2[i].i_channel = (i - channel_count) - (0);
				}
				else if (i - channel_count < ver.rx_channel_count + ver.tx_channel_count) {
					//tx channel
					chlist2[i].channel_type = T_CHANNEL_TX;
					chlist2[i].i_channel = (i - channel_count) - (ver.rx_channel_count);
				}
				else {
					//708 channel
					chlist2[i].channel_type = T_CHANNEL_708;
					chlist2[i].i_channel = (i - channel_count) - (ver.rx_channel_count + ver.tx_channel_count);
				}

				chlist2[i].g_channel = i;
				chlist2[i].b_channel = i - channel_count;

				wsize = (wcslen(devices->device[deviceCount].device_name) + 1);
				chlist2[i].device_name = (WCHAR*)malloc(sizeof(WCHAR) * wsize);
				wcsncpy_s(chlist2[i].device_name, wsize, devices->device[deviceCount].device_name, wsize);
			}
			chlist1 = chlist2;
			channel_count += ver.rx_channel_count + ver.tx_channel_count + ver.a708_channel_count;
		}

		CloseHandle(handle);

		free(devices->device[deviceCount].device_name);
		deviceCount++;
	}

	SetupDiDestroyDeviceInfoList(hdi);
	free(devices->device);

	devices->pcount = deviceCount;

	devices->count = channel_count;
	devices->device = chlist1;

	return devices;
}

VOID a708_free(A708_DEVICE_LIST* devices)
{
	for (UINT8 i = 0; i < devices->count; i++) {
		free(devices->device[i].device_name);
		if (devices->device[i].arinc != INVALID_HANDLE_VALUE) {
			CloseHandle(devices->device[i].arinc);
		}
	}
	for (UINT8 i = 0; i < devices->pcount; i++) {
		free(devices->pdev[i].device_name);
		if (devices->pdev[i].arinc != INVALID_HANDLE_VALUE) {
			CloseHandle(devices->pdev[i].arinc);
		}
	}
	if (devices->count > 0) {
		free(devices->device);
	}
	if (devices->pcount > 0) {
		free(devices->pdev);
	}
	free(devices);
}

HANDLE a708_open(A708_CHANNEL_INFO* info)
{
	HANDLE handle;

	handle = CreateFile(info->device_name,
		GENERIC_READ | GENERIC_WRITE,
		FILE_SHARE_READ | FILE_SHARE_WRITE,
		NULL,
		OPEN_EXISTING,
		0,
		NULL);

	info->arinc = handle;

	return handle;
}

UINT32 a708_read_reg(A708_CHANNEL_INFO* arinc, UINT32 addr, UINT32* pbuf)
{
	BOOL res;
	DWORD size;
	BAR_REG_REQ req;

	req.regAddr = addr;

	res = DeviceIoControl(arinc->arinc, IOCTL_READ_BAR, &req, sizeof(BAR_REG_REQ), &req, sizeof(BAR_REG_REQ), &size, NULL);
	if (!res) {
		return GetLastError();
	}

	*pbuf = req.data;

	return 0;
}

UINT32 a708_write_reg(A708_CHANNEL_INFO* arinc, UINT32 addr, UINT32 buf)
{
	BOOL res;
	DWORD size;
	BAR_REG_REQ req;

	req.regAddr = addr;
	req.data = buf;

	res = DeviceIoControl(arinc->arinc, IOCTL_WRITE_BAR, &req, sizeof(BAR_REG_REQ), NULL, 0, &size, NULL);
	if (!res) {
		return GetLastError();
	}

	return 0;
}

UINT32 a708_modify_reg(A708_CHANNEL_INFO* arinc, UINT32 addr, UINT32 buf, UINT32 mask)
{
	BOOL res;
	DWORD size;
	BAR_REG_REQ_MASK req;

	req.regAddr = addr;
	req.data = buf;
	req.mask = mask;

	res = DeviceIoControl(arinc->arinc, IOCTL_WRITE_BAR_MASK, &req, sizeof(BAR_REG_REQ_MASK), NULL, 0, &size, NULL);
	if (!res) {
		return GetLastError();
	}

	return 0;
}

UINT32 a708_read_dma(A708_CHANNEL_INFO* arinc, UINT32* pcount, DMA_BLOCK** dmabuf)
{
	BOOL res;
	DWORD size;
	DMA_REQ req;
	DMA_CONTAINER response;

	req.count = *pcount > 0 && *pcount < DMA_COUNT_BLOCKS ? *pcount : DMA_COUNT_BLOCKS;
	req.channel = arinc->b_channel;

	res = DeviceIoControl(arinc->arinc, IOCTL_READ_DMA, &req, sizeof(DMA_REQ), &response, sizeof(DMA_CONTAINER), &size, NULL);

	if (!res) {
		*pcount = 0;
		return GetLastError();
	}

	if (response.count > 0 && response.count <= DMA_COUNT_BLOCKS) {
		memcpy_s(*dmabuf, sizeof(DMA_BLOCK) * response.count, response.blocks, sizeof(DMA_BLOCK) * response.count);
		*pcount = response.count;
	}
	else {
		*pcount = 0;
	}

	return 0;
}

UINT32 a708_clear_dma(A708_CHANNEL_INFO* arinc)
{
	BOOL res;
	DWORD size;

	res = DeviceIoControl(arinc->arinc, IOCTL_CLEAN_DMA, NULL, 0, NULL, 0, &size, NULL);
	if (!res) {
		return GetLastError();
	}

	return 0;
}

UINT32 a708_get_device_info(A708_CHANNEL_INFO* arinc, VERSION_REQ* version)
{
	BOOL res;
	DWORD size;

	res = DeviceIoControl(arinc->arinc, IOCTL_VERSION, NULL, 0, version, sizeof(VERSION_REQ), &size, NULL);

	if (!res) {
		return GetLastError();
	}

	return 0;
}


UINT32 a708_manageDma(A708_CHANNEL_INFO* arinc, UINT32 value) {

	return a708_modify_reg(arinc, REG_DMA_DATA_BASE, value ? 1 : 0, 1);
}

UINT32 a708_manageChannelDma(A708_CHANNEL_INFO* arinc, UINT32 value) {

	UINT32 buf, mask, shift;

	shift = arinc->channel_type == T_CHANNEL_RX ? arinc->i_channel : arinc->i_channel + 16;
	buf = (value ? 0 : 1) << shift;
	mask = 1 << shift;

	return a708_modify_reg(arinc, REG_DMA_DISABLE, buf, mask);
}

UINT32 a708_writeRam(A708_CHANNEL_INFO* arinc, RAM_REQ* container) {
	BOOL res;
	DWORD size;

	container->channel = arinc->i_channel;

	res = DeviceIoControl(arinc->arinc, IOCTL_WRITE_RAM, container, sizeof(RAM_REQ), NULL, 0, &size, NULL);
	if (!res) {
		return GetLastError();
	}

	return 0;
}

UINT32 a708_readRam(A708_CHANNEL_INFO* arinc, RAM_REQ* container) {
	BOOL res;
	DWORD size;

	container->channel = arinc->i_channel;

	res = DeviceIoControl(arinc->arinc, IOCTL_READ_RAM, container, sizeof(RAM_REQ), container, sizeof(RAM_REQ), &size, NULL);
	if (!res) {
		return GetLastError();
	}

	return 0;
}

UINT32 a708_manageTrackingInterrupt(A708_CHANNEL_INFO* arinc, T_INT_REQ* info) {
	BOOL res;
	DWORD size;

	res = DeviceIoControl(arinc->arinc, IOCTL_TRACKING_INT, info, sizeof(T_INT_REQ), info, sizeof(T_INT_REQ), &size, NULL);
	if (!res) {
		return GetLastError();
	}

	return 0;
}

UINT32 a708_checkTrackingInterrupt(A708_CHANNEL_INFO* arinc, T_INT_REQ* info) {
	BOOL res;
	DWORD size;

	res = DeviceIoControl(arinc->arinc, IOCTL_CHECK_TRACKING_INT, info, sizeof(T_INT_REQ), info, sizeof(T_INT_REQ), &size, NULL);
	if (!res) {
		return GetLastError();
	}

	return 0;
}

UINT32 a708_txStartStop(A708_CHANNEL_INFO* arinc, UINT32 action) {
	UINT32 buf, mask;

	if (arinc->channel_type != T_CHANNEL_TX) {
		return ERROR_BAD_DEVICE;
	}

	buf = 0;
	if (action != A708_STOP) {
		buf = 1 << 2;
	}
	mask = 1 << 2;

	return a708_modify_reg(arinc, REG_TX_CONTROL(arinc->i_channel), buf, mask);
}

UINT32 a708_txStartOnce(A708_CHANNEL_INFO* arinc) {
	UINT32 buf, mask;

	if (arinc->channel_type != T_CHANNEL_TX) {
		return ERROR_BAD_DEVICE;
	}

	buf = 1 << 1;
	mask = 1 << 1;

	return a708_modify_reg(arinc, REG_TX_CONTROL(arinc->i_channel), buf, mask);
}

UINT32 a708_txSetMode(A708_CHANNEL_INFO* arinc, UINT32 mode) {
	UINT32 buf, mask;

	if (arinc->channel_type != T_CHANNEL_TX) {
		return ERROR_BAD_DEVICE;
	}

	switch (mode) {
	case A708_MODE_1:   buf = (1 << 26);
		break;
	case A708_MODE_2:   buf = (1 << 27);
		break;
	case A708_MODE_3:   buf = (1 << 26) | (1 << 27);
		break;
	default:            buf = 0;
		break;
	}

	mask = (1 << 26) | (1 << 27);

	return a708_modify_reg(arinc, REG_TX_CONF(arinc->i_channel), buf, mask);
}

UINT32 a708_writeTxFifo(A708_CHANNEL_INFO* arinc, UINT32 data) {
	if (arinc->channel_type == T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}
	return a708_write_reg(arinc, REG_TX_FIFO(arinc->i_channel), data);
}

UINT32 a708_setLabelConfReg(A708_CHANNEL_INFO* arinc, UINT32 numReg, UINT32 mask, UINT32 value) {
	UINT32 addr;

	if (arinc->channel_type != T_CHANNEL_RX) {
		return ERROR_BAD_DEVICE;
	}

	switch (numReg) {
	case 0: addr = REG_LBL_CONF_PCI_0(arinc->i_channel); break;
	case 1: addr = REG_LBL_CONF_PCI_1(arinc->i_channel); break;
	case 2: addr = REG_LBL_CONF_PCI_2(arinc->i_channel); break;
	case 3: addr = REG_LBL_CONF_PCI_3(arinc->i_channel); break;
	case 4: addr = REG_LBL_CONF_PCI_4(arinc->i_channel); break;
	case 5: addr = REG_LBL_CONF_PCI_5(arinc->i_channel); break;
	case 6: addr = REG_LBL_CONF_PCI_6(arinc->i_channel); break;
	case 7: addr = REG_LBL_CONF_PCI_7(arinc->i_channel); break;
	default: return ERROR_BAD_DEVICE;
	}

	return a708_modify_reg(arinc, addr, value, mask);
}

UINT32 a708_setSpeed(A708_CHANNEL_INFO* arinc, UINT32 speed) {
	UINT32 shift, data, mask, addr;

	if (arinc->channel_type == T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	shift = 0;
	if (arinc->channel_type == T_CHANNEL_RX) {
		addr = REG_RX_CONF(arinc->i_channel);
		shift = 6;
	}
	else {
		addr = REG_TX_CONF(arinc->i_channel);
	}

	data = 0;
	mask = (1 << (16 + shift)) | (1 << (17 + shift)) | (1 << (18 + shift));
	switch (speed) {
	case A708_SPEED_100:        data = (1 << (18 + shift));
		break;
	case A708_SPEED_12_14:      data = (1 << (17 + shift));
		break;
	case A708_SPEED_50:         data = (1 << (17 + shift)) | (1 << (18 + shift));
		break;
	case A708_SPEED_12:         data = (1 << (16 + shift));
		data |= (0xA6 << (8 + shift));
		mask |= (1 << (8 + shift)) | (1 << (9 + shift)) | (1 << (10 + shift)) | (1 << (11 + shift)) | (1 << (12 + shift)) | (1 << (13 + shift)) | (1 << (14 + shift)) | (1 << (15 + shift));
		break;
	case A708_SPEED_14_5:       data = (1 << (16 + shift));
		data |= (0x89 << (8 + shift));
		mask |= (1 << (8 + shift)) | (1 << (9 + shift)) | (1 << (10 + shift)) | (1 << (11 + shift)) | (1 << (12 + shift)) | (1 << (13 + shift)) | (1 << (14 + shift)) | (1 << (15 + shift));
		break;
	}

	return a708_modify_reg(arinc, addr, data, mask);
}

UINT32 a708_manageEnBit(A708_CHANNEL_INFO* arinc, UINT32 action) {
	UINT32 data, mask, addr;

	if (arinc->channel_type == T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	if (arinc->channel_type == T_CHANNEL_RX) {
		addr = REG_RX_CONF(arinc->i_channel);
	}
	else {
		addr = REG_TX_CONF(arinc->i_channel);
	}

	if (action == A708_START) {
		data = 1 << 31;
	}
	else {
		data = 0;
	}

	mask = 1 << 31;

	return a708_modify_reg(arinc, addr, data, mask);
}

UINT32 a708_isEnBit(A708_CHANNEL_INFO* arinc, UINT32* state) {
	UINT32 data, addr;

	if (arinc->channel_type == T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	if (arinc->channel_type == T_CHANNEL_RX) {
		addr = REG_RX_CONF(arinc->i_channel);
	}
	else {
		addr = REG_TX_CONF(arinc->i_channel);
	}

	addr = a708_read_reg(arinc, addr, &data);
	if (data & (1 << 31)) {
		*state = A708_ON;
	}
	else {
		*state = A708_OFF;
	}

	return addr;
}

UINT32 a708_parityManage(A708_CHANNEL_INFO* arinc, UINT32 parcheck, UINT32 parity) {
	UINT32 data, addr, mask;

	if (arinc->channel_type == T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	if (arinc->channel_type == T_CHANNEL_RX) {
		addr = REG_RX_CONF(arinc->i_channel);
	}
	else {
		addr = REG_TX_CONF(arinc->i_channel);
	}

	data = ((parcheck == 0) ? 0 : (1 << 30)) | ((parity == 0) ? 0 : (1 << 29));
	mask = (1 << 30) | (1 << 29);

	return a708_modify_reg(arinc, addr, data, mask);
}

UINT32 a708_rxFiltration(A708_CHANNEL_INFO* arinc, UINT32 action) {
	UINT32 data, addr, mask;

	if (arinc->channel_type == T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	if (arinc->channel_type == T_CHANNEL_RX) {
		addr = REG_RX_CONF(arinc->i_channel);
	}
	else {
		return ERROR_BAD_DEVICE;
	}

	if (action == A708_ON) {
		data = 0;
	}
	else {
		data = 1 << 1;
	}
	mask = 1 << 1;

	return a708_modify_reg(arinc, addr, data, mask);
}


UINT32 a708_rxManageFiltrationLabel(A708_CHANNEL_INFO* arinc, UINT32 label, UINT32 action) {
	UINT32 data, addr, mask;

	if (arinc->channel_type != T_CHANNEL_RX) {
		return ERROR_BAD_DEVICE;
	}

	if ((label >= 0) && (label <= 31)) {
		addr = REG_LBL_CONF_PCI_0(arinc->i_channel);
		mask = 1 << (label - 32 * 0);
		data = (action == A708_ON) ? mask : 0;
	}
	else if ((label >= 32) && (label <= 63)) {
		addr = REG_LBL_CONF_PCI_1(arinc->i_channel);
		mask = 1 << (label - 32 * 1);
		data = (action == A708_ON) ? mask : 0;
	}
	else if ((label >= 64) && (label <= 95)) {
		addr = REG_LBL_CONF_PCI_2(arinc->i_channel);
		mask = 1 << (label - 32 * 2);
		data = (action == A708_ON) ? mask : 0;
	}
	else if ((label >= 96) && (label <= 127)) {
		addr = REG_LBL_CONF_PCI_3(arinc->i_channel);
		mask = 1 << (label - 32 * 3);
		data = (action == A708_ON) ? mask : 0;
	}
	else if ((label >= 128) && (label <= 159)) {
		addr = REG_LBL_CONF_PCI_4(arinc->i_channel);
		mask = 1 << (label - 32 * 4);
		data = (action == A708_ON) ? mask : 0;
	}
	else if ((label >= 160) && (label <= 191)) {
		addr = REG_LBL_CONF_PCI_5(arinc->i_channel);
		mask = 1 << (label - 32 * 5);
		data = (action == A708_ON) ? mask : 0;
	}
	else if ((label >= 192) && (label <= 223)) {
		addr = REG_LBL_CONF_PCI_6(arinc->i_channel);
		mask = 1 << (label - 32 * 6);
		data = (action == A708_ON) ? mask : 0;
	}
	else if ((label >= 224) && (label <= 255)) {
		addr = REG_LBL_CONF_PCI_7(arinc->i_channel);
		mask = 1 << (label - 32 * 7);
		data = (action == A708_ON) ? mask : 0;
	}
	else
		return ERROR_BAD_ARGUMENTS;

	return a708_modify_reg(arinc, addr, data, mask);
}

UINT32 a708_rxClearFiltrationLabel(A708_CHANNEL_INFO* arinc) {
	UINT32 data, addr;

	if (arinc->channel_type != T_CHANNEL_RX) {
		return ERROR_BAD_DEVICE;
	}
	data = 0;

	addr = REG_LBL_CONF_PCI_0(arinc->i_channel);
	a708_write_reg(arinc, addr, data);
	addr = REG_LBL_CONF_PCI_1(arinc->i_channel);
	a708_write_reg(arinc, addr, data);
	addr = REG_LBL_CONF_PCI_2(arinc->i_channel);
	a708_write_reg(arinc, addr, data);
	addr = REG_LBL_CONF_PCI_3(arinc->i_channel);
	a708_write_reg(arinc, addr, data);
	addr = REG_LBL_CONF_PCI_4(arinc->i_channel);
	a708_write_reg(arinc, addr, data);
	addr = REG_LBL_CONF_PCI_5(arinc->i_channel);
	a708_write_reg(arinc, addr, data);
	addr = REG_LBL_CONF_PCI_6(arinc->i_channel);
	a708_write_reg(arinc, addr, data);
	addr = REG_LBL_CONF_PCI_7(arinc->i_channel);
	a708_write_reg(arinc, addr, data);

	return ERROR_SUCCESS;
}

UINT32 a708_rxManageRcvSdi(A708_CHANNEL_INFO* arinc, UINT32 action, UINT32 bit9, UINT32 bit10) {
	UINT32 data, addr, mask;

	if (arinc->channel_type == T_CHANNEL_RX) {
		addr = REG_RX_CONF(arinc->i_channel);
	}
	else {
		return ERROR_BAD_DEVICE;
	}

	mask = (1 << 2) | (1 << 25) | (1 << 26);
	data = 0;
	data |= (action == A708_OFF) ? 0 : (1 << 2);
	data |= (bit9 == A708_FALSE) ? 0 : (1 << 25);
	data |= (bit10 == A708_FALSE) ? 0 : (1 << 26);

	return a708_modify_reg(arinc, addr, data, mask);
}

UINT32 a708_deinit(A708_CHANNEL_INFO* arinc) {
	UINT32 data, addr, mask;

	if (arinc->channel_type == T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	a708_manageEnBit(arinc, A708_ON);

	if (arinc->channel_type == T_CHANNEL_TX) {
		addr = REG_TX_CONTROL(arinc->i_channel);
		mask = 1 << 1 | 1 << 2 | 1 << 3 | 1 << 8 | 1 << 9 | 1 << 10 | 1 << 11 | 1 << 12 | 1 << 13 | 1 << 14 | 1 << 15 | 1 << 16;
		data = 1 << 3;
		a708_modify_reg(arinc, addr, data, mask);
	}

	a708_manageEnBit(arinc, A708_OFF);
	if (arinc->channel_type == T_CHANNEL_RX) {
		addr = REG_RX_CONF(arinc->i_channel);
		data = 1 << 1;
	}
	else {
		addr = REG_TX_CONF(arinc->i_channel);
		data = 0;
	}

	return a708_write_reg(arinc, addr, data);
}

UINT32 a708_manageReverce(A708_CHANNEL_INFO* arinc, UINT32 action) {
	UINT32 data, addr, mask;

	if (arinc->channel_type == T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	if (arinc->channel_type != T_CHANNEL_TX) {
		addr = REG_RX_CONF(arinc->i_channel);
	}
	else {
		addr = REG_TX_CONF(arinc->i_channel);
	}
	data = (action == 0) ? 0 : (1 << 28);
	mask = 1 << 28;
	return a708_modify_reg(arinc, addr, data, mask);
}

UINT32 a708_txGapBits(A708_CHANNEL_INFO* arinc, UINT32 gapBit) {
	UINT32 data, addr, mask;

	if (arinc->channel_type != T_CHANNEL_TX) {
		return ERROR_BAD_DEVICE;
	}
	else {
		addr = REG_TX_CONF(arinc->i_channel);
	}

	mask = (1 << 0) | (1 << 1) | (1 << 2) | (1 << 3) | (1 << 4) | (1 << 5) | (1 << 6);
	gapBit &= mask;
	mask = mask << 19;
	data = gapBit << 19;

	return a708_modify_reg(arinc, addr, data, mask);
}

UINT32 a708_txRrD(A708_CHANNEL_INFO* arinc, UINT32 txrr) {
	UINT32 data, addr, mask;
	if (arinc->channel_type != T_CHANNEL_TX) {
		return ERROR_BAD_DEVICE;
	}
	else {
		addr = REG_TX_CONTROL(arinc->i_channel);
	}

	mask = 1 << 16;
	data = (txrr == A708_TXRR_10MS) ? 0 : 1 << 16;

	return a708_modify_reg(arinc, addr, data, mask);
}

UINT32 a708_txSkipRrt(A708_CHANNEL_INFO* arinc, UINT32 action) {
	UINT32 data, addr, mask;
	if (arinc->channel_type != T_CHANNEL_TX) {
		return ERROR_BAD_DEVICE;
	}
	else {
		addr = REG_TX_CONTROL(arinc->i_channel);
	}

	mask = 1 << 3;
	data = (action == A708_OFF) ? 0 : 1 << 3;

	return a708_modify_reg(arinc, addr, data, mask);
}

UINT32 a708_manageScOut(A708_CHANNEL_INFO* arinc, UINT32 action1, UINT32 action2, UINT32 action3, UINT32 action4/*, UINT32 action5, UINT32 action6, UINT32 action7, UINT32 action8*/) {
	UINT32 data, addr, mask;

	if (arinc->channel_type == T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	addr = REG_SC_OUT;
	mask = 0xff; //ffff
	data = 0;
	data |= (action1 == A708_SC_OUT_0) ? (A708_SC_OUT_0 << 0) : ((action1 == A708_SC_OUT_1) ? (A708_SC_OUT_1 << 0) : A708_SC_OUT_NO_ACT);
	data |= (action2 == A708_SC_OUT_0) ? (A708_SC_OUT_0 << 2) : ((action2 == A708_SC_OUT_1) ? (A708_SC_OUT_1 << 2) : A708_SC_OUT_NO_ACT);
	data |= (action3 == A708_SC_OUT_0) ? (A708_SC_OUT_0 << 4) : ((action3 == A708_SC_OUT_1) ? (A708_SC_OUT_1 << 4) : A708_SC_OUT_NO_ACT);
	data |= (action4 == A708_SC_OUT_0) ? (A708_SC_OUT_0 << 6) : ((action4 == A708_SC_OUT_1) ? (A708_SC_OUT_1 << 6) : A708_SC_OUT_NO_ACT);
	/*
	data |= (action5 == A429_SC_OUT_0) ? (A429_SC_OUT_0 << 8) : ((action5 == A429_SC_OUT_1) ? (A429_SC_OUT_1 << 8) : A429_SC_OUT_NO_ACT);
	data |= (action6 == A429_SC_OUT_0) ? (A429_SC_OUT_0 << 10) : ((action6 == A429_SC_OUT_1) ? (A429_SC_OUT_1 << 10) : A429_SC_OUT_NO_ACT);
	data |= (action7 == A429_SC_OUT_0) ? (A429_SC_OUT_0 << 12) : ((action7 == A429_SC_OUT_1) ? (A429_SC_OUT_1 << 12) : A429_SC_OUT_NO_ACT);
	data |= (action8 == A429_SC_OUT_0) ? (A429_SC_OUT_0 << 14) : ((action8 == A429_SC_OUT_1) ? (A429_SC_OUT_1 << 14) : A429_SC_OUT_NO_ACT);
	*/
	return a708_modify_reg(arinc, addr, data, mask);
}

UINT32 a708_clearScIntMask(A708_CHANNEL_INFO* arinc) {
	UINT32 data, addr;
	if (arinc->channel_type == T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}
	addr = REG_SC_INT_MASK;
	data = 0;
	return a708_write_reg(arinc, addr, data);
}

#define A708_MAN_SC_INT_MASK_SET(num, rise, fall) \
    case A708_IN_RK_##num: shift = (stateRk == A708_SC_RISE) ? rise: fall; \
        break

UINT32 a708_manageScIntMask(A708_CHANNEL_INFO* arinc, UINT32 numRk, UINT32 stateRk, UINT32 actionRk) {
	UINT32 data, addr, mask, shift;
	data = 0;
	mask = 0;
	addr = REG_SC_INT_MASK;
	shift = 0;

	if (arinc->channel_type == T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	if (numRk > 4) {
		return ERROR_BAD_ARGUMENTS;
	}

	switch (numRk) {
		A708_MAN_SC_INT_MASK_SET(1, 4, 0);
		A708_MAN_SC_INT_MASK_SET(2, 5, 1);
		A708_MAN_SC_INT_MASK_SET(3, 6, 2);
		A708_MAN_SC_INT_MASK_SET(4, 7, 3);
		/*A429_MAN_SC_INT_MASK_SET(5, 12, 8);
		A429_MAN_SC_INT_MASK_SET(6, 13, 9);
		A429_MAN_SC_INT_MASK_SET(7, 14, 10);
		A429_MAN_SC_INT_MASK_SET(8, 15, 11);*/
	}
	mask = 1 << shift;
	if (actionRk == A708_ON) {
		data = mask;
	}
	return a708_modify_reg(arinc, addr, data, mask);
}

UINT32 a708_txStartWithRrt(A708_CHANNEL_INFO* arinc, UINT32 rrtValue) {
	UINT32 maskRrt, data, addr, mask;

	if (arinc->channel_type != T_CHANNEL_TX) {
		return ERROR_BAD_DEVICE;
	}
	else {
		addr = REG_TX_CONTROL(arinc->i_channel);
	}

	maskRrt = (1 << 8) | (1 << 9) | (1 << 10) | (1 << 11) | (1 << 12) | (1 << 13) | (1 << 14) | (1 << 15);
	data = (rrtValue << 8) & maskRrt;
	mask = maskRrt | (1 << 2);
	data |= (1 << 2);

	return a708_modify_reg(arinc, addr, data, mask);
}

UINT32 a708_txStartWithRrtOnce(A708_CHANNEL_INFO* arinc, UINT32 rrtValue) {
	UINT32 maskRrt, data, addr, mask;

	if (arinc->channel_type != T_CHANNEL_TX) {
		return ERROR_BAD_DEVICE;
	}
	else {
		addr = REG_TX_CONTROL(arinc->i_channel);
	}

	maskRrt = (1 << 8) | (1 << 9) | (1 << 10) | (1 << 11) | (1 << 12) | (1 << 13) | (1 << 14) | (1 << 15);
	data = (rrtValue << 8) & maskRrt;
	mask = maskRrt | (1 << 1);
	data |= (1 << 1);

	return a708_modify_reg(arinc, addr, data, mask);
}

UINT32 a708_txSetRequestTransferByRkIn(A708_CHANNEL_INFO* arinc, UINT32 rknum) {
	BOOL res;
	DWORD size;
	T_INT_REQ in;

	if (arinc->channel_type != T_CHANNEL_TX) {
		return ERROR_BAD_DEVICE;
	}
	if (rknum > RKIN_NUM_4) {
		rknum = RKIN_NONE;
	}

	in.channel = arinc->b_channel;
	in.manage = rknum;

	res = DeviceIoControl(arinc->arinc, IOCTL_START_AS_RK_IN, &in, sizeof(T_INT_REQ), NULL, 0, &size, NULL);
	if (!res) {
		return GetLastError();
	}

	return 0;
}

UINT32 a708_manageInterruptMask(A708_CHANNEL_INFO* arinc, UINT32 mask, UINT32 value) {
	return a708_modify_reg(arinc, REG_INTERRUPT_MASK, value, mask);
}

UINT32 a708_rxSetReceiveReadyByRkIn(A708_CHANNEL_INFO* arinc, UINT32 rknum) {
	BOOL res;
	DWORD size;
	T_INT_REQ in;

	if (arinc->channel_type != T_CHANNEL_RX) {
		return ERROR_BAD_DEVICE;
	}
	if (rknum > RKIN_NUM_4) {
		rknum = RKIN_NONE;
	}

	in.channel = arinc->b_channel;
	in.manage = rknum;

	res = DeviceIoControl(arinc->arinc, IOCTL_START_AS_RK_IN, &in, sizeof(T_INT_REQ), NULL, 0, &size, NULL);
	if (!res) {
		return GetLastError();
	}

	return 0;
}

UINT32 a708_ramCls(A708_CHANNEL_INFO* arinc) {
	UINT32 data, addr, mask;

	if (arinc->channel_type == T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	if (arinc->channel_type == T_CHANNEL_TX) {
		addr = REG_TX_CONF(arinc->i_channel);
	}
	else {
		addr = REG_RX_CONF(arinc->i_channel);
	}

	mask = 1;
	data = 1;

	return a708_modify_reg(arinc, addr, data, mask);
}

UINT32 a708_rx708_manage(A708_CHANNEL_INFO* arinc, UINT32 mode) {
	UINT32 addr;
	UINT32 buf;
	UINT32 mask;

	if (arinc->channel_type != T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	addr = REG_RX_CONF_708(arinc->i_channel);
	mask = 1 << 1;
	buf = (mode == A708_OFF) ? 1 << 1 : 0;

	return a708_modify_reg(arinc, addr, buf, mask);
}

UINT32 a708_tx708_setMode(A708_CHANNEL_INFO* arinc, UINT32 mode) {
	UINT32 addr;
	UINT32 buf;
	UINT32 mask;

	if (arinc->channel_type != T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	addr = REG_TX_CONF_708(arinc->i_channel);
	mask = 1 << 1 | 1;
	switch (mode) {
	case A708_TX708_MODE_SINGLE_SHOT:
		buf = A708_TX708_MODE_SINGLE_SHOT;
		break;
	case A708_TX708_MODE_AUTO:
		buf = A708_TX708_MODE_AUTO;
		break;
	default:
		buf = 0;
	}

	return a708_modify_reg(arinc, addr, buf, mask);
}

UINT32 a708_rx708_shortDma(A708_CHANNEL_INFO* arinc, UINT32 mode) {
	UINT32 addr;
	UINT32 buf;
	UINT32 mask;

	if (arinc->channel_type != T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	addr = REG_RX_CONF_708(arinc->i_channel);
	mask = 1;
	buf = (mode == A708_OFF) ? 0 : 1;

	return a708_modify_reg(arinc, addr, buf, mask);
}

UINT32 a708_readDmaA708(A708_CHANNEL_INFO* arinc, UINT32* pcount, DMA_BLOCK_708** dmabuf) {
	BOOL res;
	DWORD size;
	DMA_REQ req;
	DMA_CONTAINER_708 response;

	if (arinc->channel_type != T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	req.count = *pcount > 0 && *pcount < DMA_COUNT_BLOCKS_708 ? *pcount : DMA_COUNT_BLOCKS_708;
	req.channel = arinc->b_channel;

	res = DeviceIoControl(arinc->arinc, IOCTL_READ_DMA_708, &req, sizeof(DMA_REQ), &response, sizeof(DMA_CONTAINER_708), &size, NULL);

	if (!res) {
		*pcount = 0;
		return GetLastError();
	}

	if (response.count > 0 && response.count <= DMA_COUNT_BLOCKS_708) {
		memcpy_s(*dmabuf, sizeof(DMA_BLOCK_708) * response.count, response.blocks, sizeof(DMA_BLOCK_708) * response.count);
		*pcount = response.count;
	}
	else {
		*pcount = 0;
	}

	return 0;
}

UINT32 a708_tx708_writeDataBlock(A708_CHANNEL_INFO* arinc, UINT32 numBlock, T_A708_DATA_BLOCK* block) {
	UINT32 addr;
	UINT32 buf;
	UINT32 ret;
	if (arinc->channel_type != T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	switch (numBlock) {
	case 1: addr = REG_TX708_DAT_BLOCK1(arinc->i_channel); break;
	case 2: addr = REG_TX708_DAT_BLOCK2(arinc->i_channel); break;
	case 3: addr = REG_TX708_DAT_BLOCK3(arinc->i_channel); break;
	case 4: addr = REG_TX708_DAT_BLOCK4(arinc->i_channel); break;
	case 5: addr = REG_TX708_DAT_BLOCK5(arinc->i_channel); break;
	case 6: addr = REG_TX708_DAT_BLOCK6(arinc->i_channel); break;
	case 7: addr = REG_TX708_DAT_BLOCK7(arinc->i_channel); break;
	case 8: addr = REG_TX708_DAT_BLOCK8(arinc->i_channel); break;
	default:
		return ERROR_INVALID_PARAMETER;
	}

	if (block->length > A708_DATA_BLOCK_SIZE_708) {
		return ERROR_INVALID_PARAMETER;
	}

	for (int i = 0; i < block->length; i++) {
		buf = block->words[i];
		ret = a708_write_reg(arinc, addr, buf);
		addr += 0x4;
		if (ret != ERROR_SUCCESS)
			return ret;
	}

	return ERROR_SUCCESS;
}

UINT32 a708_tx708_readDataBlock(A708_CHANNEL_INFO* arinc, UINT32 numBlock, T_A708_DATA_BLOCK* block) {
	UINT32 addr;
	UINT32 buf;
	UINT32 ret;
	if (arinc->channel_type != T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	switch (numBlock) {
	case 1: addr = REG_TX708_DAT_BLOCK1(arinc->i_channel); break;
	case 2: addr = REG_TX708_DAT_BLOCK2(arinc->i_channel); break;
	case 3: addr = REG_TX708_DAT_BLOCK3(arinc->i_channel); break;
	case 4: addr = REG_TX708_DAT_BLOCK4(arinc->i_channel); break;
	case 5: addr = REG_TX708_DAT_BLOCK5(arinc->i_channel); break;
	case 6: addr = REG_TX708_DAT_BLOCK6(arinc->i_channel); break;
	case 7: addr = REG_TX708_DAT_BLOCK7(arinc->i_channel); break;
	case 8: addr = REG_TX708_DAT_BLOCK8(arinc->i_channel); break;
	default:
		return ERROR_INVALID_PARAMETER;
	}

	if (block->length > A708_DATA_BLOCK_SIZE_708) {
		return ERROR_INVALID_PARAMETER;
	}

	for (int i = 0; i < block->length; i++) {
		ret = a708_read_reg(arinc, addr, &buf);
		block->words[i] = buf;
		addr += 0x4;
		if (ret != ERROR_SUCCESS)
			return ret;
	}

	return ERROR_SUCCESS;
}

UINT32 a708_tx708_start(A708_CHANNEL_INFO* arinc, UINT32 mode)
{
	UINT32 addr;
	UINT32 buf;

	if (arinc->channel_type != T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	addr = REG_TX_CONF_708(arinc->i_channel);

	switch (mode) {
	case A708_TX708_MODE_SINGLE_SHOT:
		buf = A708_TX708_MODE_SINGLE_SHOT | 1 << 2;
		break;
	case A708_TX708_MODE_AUTO:
		buf = A708_TX708_MODE_AUTO | 1 << 2;
		break;
	default:
		buf = 0;
		break;
	}
	return a708_write_reg(arinc, addr, buf);
}

UINT32 a708_tx708_hasSendedDataBlock(A708_CHANNEL_INFO* arinc, UINT32 block, UINT32* result) {
	UINT32 addr;
	UINT32 data;
	UINT32 res;
	if (arinc->channel_type != T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	if (block > 8 || block == 0) {
		return ERROR_INVALID_PARAMETER;
	}

	data = 0;
	addr = REG_TX_CONF_708(arinc->i_channel);

	res = a708_read_reg(arinc, addr, &data);

	*result = (data & (1 << (7 + block)) > 0) ? A708_FALSE : A708_TRUE;

	return res;
}

UINT32 a708_tx708_stop(A708_CHANNEL_INFO* arinc) {
	UINT32 addr;
	UINT32 data;
	if (arinc->channel_type != T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	data = 0;
	addr = REG_TX_CONF_708(arinc->i_channel);

	return a708_write_reg(arinc, addr, data);
}

UINT32 a708_tx708_writeTimer(A708_CHANNEL_INFO* arinc, UINT32 value) {

	UINT32 addr;
	UINT32 data;
	if (arinc->channel_type != T_CHANNEL_708) {
		return ERROR_BAD_DEVICE;
	}

	data = value;
	addr = REG_TX_TIME_708(arinc->i_channel);

	return a708_write_reg(arinc, addr, data);

}